
def MinerTransaction():
    """

    :return:
    """
    return b'\x00'


def IssueTransaction():
    """

    :return:
    """
    return b'\x01'


def ClaimTransaction():
    """

    :return:
    """
    return b'\x02'


def EnrollmentTransaction():
    """

    :return:
    """
    return b'\x20'


def VotingTransaction():
    """

    :return:
    """
    return b'\x24'


def RegisterTransaction():
    """

    :return:
    """
    return b'\x40'


def ContractTransaction():
    """

    :return:
    """
    return b'\x80'


def AgencyTransaction():
    """

    :return:
    """
    return b'\xb0'


def PublishTransaction():
    """

    :return:
    """
    return b'\xd0'


def InvocationTransaction():
    """

    :return:
    """
    return b'\xd1'
