
def Main():

    # lets have fun

    # this dosen't work. sad
    #        m = [[1,2,3],'fun','cool','neo']

    """

    :return:
    """
    m = [42, 'yay', 'neo', 'woolong', 'wtf-coin', b'\xff', 3, 10]

    answer = m[0]

    return answer
