

def GetTrigger():
    """

    """
    pass


def CheckWitness(hash_or_pubkey):
    """

    :param hash_or_pubkey:
    """
    pass


def Log(message):
    """

    :param message:
    """
    pass


def Notify(arg):
    """

    :param arg:
    """
    pass
