
class TransactionInput():

    @property
    def Hash(self):
        """

        :return:
        """
        return GetHash(self)

    @property
    def Index(self):
        """

        :return:
        """
        return GetIndex(self)


def GetHash(input):
    """

    :param input:
    """
    pass


def GetIndex(input):
    """

    :param input:
    """
    pass
