
def Main(a, b, c, d):

    """

    :param a:
    :param b:
    :param c:
    :param d:
    :return:
    """
    j = a & b

    e = c / d

    f = a - c

    g = f * j

    k = e % d

    l = g ^ c

    m = j + e + f + g + l

    return m
